/****************************************************************************
 *                                                                          
 * Title: Example of S-parameter measurement
 *
 * Note: This example is designed for use with LabVINDOWS/CVI 7.0 or later
 *
 * Purpose: This measurement example shows basic principles of how to work
 *          with the ZVB instrument driver.
 *
 ****************************************************************************/

/* --- LabVINDOWS/CVI specific libraries --- */
#include <userint.h>
#include <utility.h>
/* --- Common C libraries, Instrument driver library, VISA library (included in instrument driver) --- */
#include <ansi_c.h>
#include <visa.h>
#include "rszvb.h"
#include "RSZVB_Measurement_Example.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        char    BuFfEr[512];\
        error = (fCal);\
        rszvb_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (BuFfEr, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", BuFfEr); \
            rszvb_error_query (io, &error, error_message);\
            sprintf (BuFfEr, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            MessagePopup ("Error", BuFfEr); \
            runState = 0; \
            HidePanel (panelHandle[MEAS]); \
            DisplayPanel (panelHandle[CONFIG]); \
            break; \
        } \
    }

/* Globals */

ViInt32     runState            = 0,
            uirCode             = 0,
            panelHandle[10];

/* Define remote connection parameters (as default values) */

ViStatus    error               = VI_SUCCESS;
ViChar      error_message[512];
ViSession   io;

/*=================== Utility Function Declarations =========================*/

long abortApplication (void);

/*= Main Function ===========================================================*/

void main ()
{
    /* Load panels.*/
    panelHandle[BCKGRND] = LoadPanel (panelHandle[BCKGRND], "RSZVB_Measurement_Example.uir", BCKGRND);
    panelHandle[MEAS] = LoadPanel (panelHandle[BCKGRND], "RSZVB_Measurement_Example.uir", MEAS);
    panelHandle[CONFIG] = LoadPanel (panelHandle[BCKGRND], "RSZVB_Measurement_Example.uir", CONFIG); 
    panelHandle[INIT] = LoadPanel (panelHandle[BCKGRND], "RSZVB_Measurement_Example.uir", INIT);
    /* Display panels.*/
    DisplayPanel (panelHandle[BCKGRND]);
    DisplayPanel (panelHandle[INIT]);
    SetSleepPolicy (VAL_SLEEP_MORE);
    SetBreakOnLibraryErrors (0);

    uirCode = RunUserInterface ();
}   

/*===========================================================================*/
/* Function: Initialize Instrument                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Initialize panel. It initializes the instrument and switches to */
/*           the panel Configure.                                            */
/*===========================================================================*/
int CVICALLBACK initInstrument (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar instrDescr[256], errorMessage[256], errBuffer[256];
    ViInt16 rst, id;

    switch (event) {
        case EVENT_COMMIT:
            SetWaitCursor(0);
            GetCtrlVal (panelHandle[INIT], INIT_ADDRESS, instrDescr);
            GetCtrlVal (panelHandle[INIT], INIT_ID, &id);
            GetCtrlVal (panelHandle[INIT], INIT_RST, &rst);
            SetWaitCursor(1);
            if ((error = rszvb_init (instrDescr, id, rst, &io)) < 0 )
                {
                rszvb_error_message(VI_NULL, error, errorMessage);
                sprintf (errBuffer, "Initialization Error:\n\n%s\n\nCheck your "
                        "connections and make sure you have the right instrument descriptor.",
                         errorMessage);
                MessagePopup ("ERROR!", errBuffer);
                SetWaitCursor(0);
                return(0);   
                }

            /* --- */

            CHECKERR (rszvb_SetDisplayUpdate (io, RSZVB_DISP_UPDATE_ON));
            CHECKERR (viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000));

            /* --- */

            SetWaitCursor(0);
            HidePanel (panelHandle[INIT]);
            DisplayPanel (panelHandle[CONFIG]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button causes the instrument to be initialized.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Configure Measurement & Measure                                 */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Configure panel (configure & measure).                          */
/*===========================================================================*/
int CVICALLBACK configMeasurement (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            {
            ViInt32     channel         = 1,
                        Sin             = 1,
                        Sout            = 2,
                        sweep_type      = RSZVB_SWEEP_LIN,
                        no_ofValues     = 0,
                        window          = 1,
                        windowTrace     = 2;
            ViReal64    startFrequency  = 300.0e3,
                        stopFrequency   = 4.0e9,
                        CWFrequency     = 1.0e9,
                        startPower      = 300.0e3,
                        stopPower       = 4.0e9,
                        power           = 0.0;
            ViString    traceName       = "TestTrace",
                        setupName       = "TestSetup";
            ViReal64    stimulusData[20001],
                        responseData[20001];
            ViChar      catalog[1024],
                        path[256],
                        *p2buf;
        
            /* --- */

            runState = 0;
            SetWaitCursor(0);
            HidePanel (panelHandle[CONFIG]);
            DisplayPanel (panelHandle[MEAS]);
            DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_IMMEDIATE_DRAW);

            SetWaitCursor(1);

            /* Window New */
            CHECKERR (rszvb_WindowNew (io, setupName));
            /* Remove all possibly opened traces */
            CHECKERR (rszvb_TraceList (io, channel, catalog, 1024));

            p2buf = strtok(catalog, ",");

            while(p2buf)
                {
                CHECKERR (rszvb_TraceDelete (io, channel, p2buf));
                /* Skip the meas parameter string, only trace name is important */
                p2buf = strtok(NULL, ",");
                if (p2buf)
                    p2buf = strtok(NULL, ",");
                }

            /* Trace Add */
            CHECKERR (rszvb_TraceAdd (io, channel, "TestTrace"));
            /* Trace Assign Diagram Area */
            CHECKERR (rszvb_TraceAssignDiagramArea (io, window, windowTrace, traceName));
            /* Set Sweep Number Of Points */
            CHECKERR (rszvb_SetSweepNumberOfPoints (io, channel, 201));
            /* Set Sweep Single */
            CHECKERR (rszvb_SetSweepSingle (io, channel, RSZVB_SWEEP_SINGLE));
            /* Set Sweep Count */
            CHECKERR (rszvb_SetSweepCount (io, channel, 1));

            /* Set Sweep Type */
            GetCtrlVal (panelHandle[CONFIG], CONFIG_SWEEP_TYPE, &sweep_type);
            CHECKERR (rszvb_SetSweepType (io, channel, sweep_type));

            if (sweep_type == RSZVB_SWEEP_LIN)
                {
                /* Set Start Frequency */
                GetCtrlVal (panelHandle[CONFIG], CONFIG_FREQ_START, &startFrequency);
                CHECKERR (rszvb_SetStartFrequency (io, channel, startFrequency));
                /* Set Stop Frequency */
                GetCtrlVal (panelHandle[CONFIG], CONFIG_FREQ_STOP, &stopFrequency);
                CHECKERR (rszvb_SetStopFrequency (io, channel, stopFrequency));
                /* Set Power */
                GetCtrlVal (panelHandle[CONFIG], CONFIG_SOURCE_PWR, &power);
                CHECKERR (rszvb_SetPower (io, channel, power));
                }
            else
                {
                /* Set CW Frequency */
                GetCtrlVal (panelHandle[CONFIG], CONFIG_FREQ_CW, &CWFrequency);
                CHECKERR (rszvb_SetCWFrequency (io, channel, CWFrequency));
                /* Set Start Power */
                GetCtrlVal (panelHandle[CONFIG], CONFIG_PWR_START, &startPower);
                CHECKERR (rszvb_SetStartPower (io, channel, startPower));
                /* Set Stop Power */
                GetCtrlVal (panelHandle[CONFIG], CONFIG_PWR_STOP, &stopPower);
                CHECKERR (rszvb_SetStopPower (io, channel, stopPower));
                }

            /* Select S-Parameters */
            GetCtrlVal (panelHandle[CONFIG], CONFIG_S_IN, &Sin);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_S_OUT, &Sout);
            CHECKERR (rszvb_SelectSParameters (io, channel, traceName, Sout, Sin));
            /* Set Trace Format */
            CHECKERR (rszvb_SetTraceFormat (io, channel, RSZVB_DB_MAG));
            /* Send Channel Trigger and Wait for OPC */
            {
                ViInt32 timeout = 1000;

                CHECKERR (viGetAttribute (io, VI_ATTR_TMO_VALUE, &timeout));
                CHECKERR (rszvb_SendChannelTriggerWaitOPC (io, channel, timeout));
            }
            /* Trace Autoscale */
            CHECKERR (rszvb_TraceAutoscale (io, window, windowTrace));
            /* Trace Stimulus Data */
            CHECKERR (rszvb_TraceStimulusData (io, channel, &no_ofValues, stimulusData));
            /* Trace Response Data */
            CHECKERR (rszvb_TraceResponseData (io, channel, RSZVB_FORMATTED, &no_ofValues, responseData));
            
            /* Export Trace Data to File */
            GetCtrlVal (panelHandle[CONFIG], CONFIG_EXPORT_PATH, path);
            CHECKERR (rszvb_GetCurrentDirectory (io, catalog));
            
            p2buf = catalog + strlen(catalog);
            if (strrchr (path, '\\'))
                {
                sprintf (p2buf, "%s", strrchr (path, '\\'));
                }
            else
                {
                sprintf (p2buf, "\\%s", path);
                }
            
            CHECKERR (rszvb_TraceExportData (io, traceName, catalog));
            CHECKERR (rszvb_readToFile (io, catalog, path));
            CHECKERR (rszvb_FileManager (io, RSZVB_FILE_MAN_DELETE, catalog, ""));

            /* --- */

            DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_DELAYED_DRAW);
            PlotXY (panelHandle[MEAS], MEAS_GRAPH, stimulusData, responseData,
                    no_ofValues, VAL_DOUBLE, VAL_DOUBLE, VAL_THIN_LINE,
                    VAL_SIMPLE_DOT, VAL_SOLID, 1, VAL_YELLOW);

            /* --- */
    
            /* Window Close */
            // CHECKERR (rszvb_WindowClose (io, setupName));

            /* --- */

            runState = 1;
            SetWaitCursor(0);
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will configure the instrument to take a measurement.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Sweep Type                                                      */
/* Purpose:  Enables or disables sets of values for selected sweep type.     */
/*===========================================================================*/
int CVICALLBACK sweepType(int panel, int control, int event, void *callbackData,
        int eventData1, int eventData2)
{
    ViInt32 sweep_type = 0;
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetCtrlVal (panelHandle[CONFIG], CONFIG_SWEEP_TYPE, &sweep_type);
            if (sweep_type == RSZVB_SWEEP_LIN)
                {
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_FREQ_CW, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_PWR_START, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_PWR_STOP, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_SOURCE_PWR, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_FREQ_START, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_FREQ_STOP, ATTR_DIMMED, 0);
                }
            else
                {
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_FREQ_CW, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_PWR_START, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_PWR_STOP, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_SOURCE_PWR, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_FREQ_START, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle[CONFIG], CONFIG_FREQ_STOP, ATTR_DIMMED, 1);
                }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help", "Selects the sweep type, i.e. the sweep variable (frequency/power)\n"
                                  " and the position of the sweep points across the sweep range.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Cancel                                                          */
/* Purpose:  Called by the init panel this function pops up a confirmation   */
/*           dialog box and then quits the user interface, if desired.       */
/*===========================================================================*/
int CVICALLBACK Cancel (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    SetWaitCursor(0);
    switch (event)
        {
        case EVENT_COMMIT:
            if ((ConfirmPopup ("Exit Application", "Are you sure you want"
                                                   " to quit this application?")) == 1)
                {
                QuitUserInterface (uirCode);
                }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Control Help", "Closes the application.");
            break;
        }

    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls that configure the  */
/*           instrument. On the right mouse-click on the control a help      */
/*           window describing its purpose is displayed.                     */
/*===========================================================================*/
int CVICALLBACK controlHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{   
    SetWaitCursor(0);

    switch (event)
        {
        case EVENT_RIGHT_CLICK:
            
            if (panel == panelHandle[INIT])
                switch (control)
                    {
                    case INIT_ADDRESS:
                        MessagePopup ("Help", "This control specifies the interface and address of the device\n"
                                              "that is to be initialized (Instrument Descriptor). The exact\n"
                                              "grammar to be used in this control is shown in the note below.\n\n"
                                              "Default Value: GPIB::20\n\n"
                                              "Notes:\n\n"
                                              "(1) Based on the Instrument Descriptor, this operation\n"
                                              "establishes a communication session with a device.  The grammar\n"
                                              "for the Instrument Descriptor is shown below.  Optional\n"
                                              "parameters are shown in square brackets ([]).\n\n"
                                              "Interface   Grammar\n"
                                              "------------------------------------------------------\n"
                                              "Serial      ASRL<port>[::INSTR]\n"
                                              "GPIB        GPIB[board]::primary address[::secondary address]\n"
                                              "            [::INSTR]\n"
                                              "RSIB        RSIB::remote_host::INSTR\n"
                                              "VXI-11      TCPIP[board]::host address[::LAN device name][::INSTR]\n\n"
                                              "The ASRL keyword is used for serial interface.\n"
                                              "The GPIB keyword is used for GPIB interface.\n"
                                              "The RSIB keyword is used for RSIB interface.\n"
                                              "The VXI-11 keyword is used for a TCP/IP device located at the specified address.\n\n"
                                              "Examples:\n"
                                              "(1) Serial - ASRL1\n"
                                              "(2) GPIB   - GPIB::20\n"
                                              "(3) RSIB   - RSIB::192.168.1.33::INSTR\n"
                                              "(4) VXI-11 - TCPIP::10.0.0.123::INSTR\n\n"
                                              "The default value for optional parameters are shown below.\n\n"
                                              "Optional Parameter          Default Value\n"
                                              "-----------------------------------------\n"
                                              "board                       0\n"
                                              "secondary address           none - 31\n\n"
                                              "Note:\n\n"
                                              "(1) Parameters of the serial interface, you must set via T&M\n"
                                              "Expolrer before you call initialize function. Default values are\n"
                                              "Baud Rate: 9600, Data Bits: 8, Parity: None, Stop Bits: 1 and\n"
                                              "Flow Control: None");
                        break;
                    case INIT_ID:
                        MessagePopup ("Help", "This control specifies if an ID Query is sent to the\n"
                                              "instrument during the initialization procedure.\n\n"
                                              "Valid Range:\n"
                                              "0 - Skip Query\n"
                                              "1 - Do Query (Default Value)");
                        break;
                    case INIT_RST:
                        MessagePopup ("Help", "This control specifies if the instrument is to be reset to its\n"
                                              "power-on settings during the initialization procedure.\n\n"
                                              "Valid Range:\n"
                                              "0 - Don't Reset\n"
                                              "1 - Reset Device (Default Value)");
                        break;
                    }

            if (panel == panelHandle[CONFIG])
                switch (control)
                    {
                    case CONFIG_S_IN:
                        MessagePopup ("Help", "S-parameters are expressed as S<out><in>, where <out> and <in>\n"
                                              "denote the output and input port numbers of the DUT.\n\n"
                                              "Valid Range: 1 to 4");
                        break;
                    case CONFIG_S_OUT:
                        MessagePopup ("Help", "S-parameters are expressed as S<out><in>, where <out> and <in>\n"
                                              " denote the output and input port numbers of the DUT.\n\n"
                                              "Valid Range: 1 to 4");
                        break;
                    case CONFIG_EXPORT_PATH:
                        MessagePopup ("Help", "String parameter to specify the name and directory of the\n"
                                              "created trace file on your local hard drive. Several file\n"
                                              "formats for trace files are supported.\n\n"
                                              "Note:\n\n"
                                              "Trace files are ASCII files with selectable file format:\n\n"
                                              " - Touchstone (*.s<n>p) files\n"
                                              " - ASCII (*.csv) files\n"
                                              " - Mathlab (*.dat) files");
                        break;
                    case CONFIG_FREQ_CW:
                        MessagePopup ("Help", "This control defines the fixed (Continuous Wave, CW) frequency for\n"
                                              "all sweep types operating at fixed frequency (power sweep, time sweep,\n"
                                              "CW mode sweep).\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    case CONFIG_PWR_START:
                        MessagePopup ("Help", "This control defines the start power for a power sweep which is equal\n"
                                              "to the left edge of a Cartesian diagram.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case CONFIG_PWR_STOP:
                        MessagePopup ("Help", "This control defines the stop power for a power sweep which is equal\n"
                                              "to the right edge of a Cartesian diagram.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case CONFIG_SOURCE_PWR:
                        MessagePopup ("Help", "This control defines the power of the internal signal source.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case CONFIG_FREQ_START:
                        MessagePopup ("Help", "This control defines the start frequency for a frequency sweep\n"
                                              "which is equal to the left edge of a Cartesian diagram.\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    case CONFIG_FREQ_STOP:
                        MessagePopup ("Help", "This control defines the stop frequency for a frequency sweep\n"
                                              "which is equal to the right edge of a Cartesian diagram.\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    }

            if (panel == panelHandle[MEAS])
                switch (control)
                    {
                    case MEAS_GRAPH:
                        MessagePopup ("Help", "Display of measurement results.");
                        break;
                    }

            break;
        }
    
    return 0;
}

/*===========================================================================*/
/* Function: Launch Configure Panel                                          */
/* Purpose:  This is a callback function of the button Configure on the      */
/*           panel Measure. It returns back to the Configuration panel to be */
/*           able to change configuration without having to re-run this      */
/*           program.                                                        */
/*===========================================================================*/
int CVICALLBACK launchConfig (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            runState = 0;
            HidePanel (panelHandle[MEAS]);
            DisplayPanel (panelHandle[CONFIG]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will recall the configuration panel.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    SetWaitCursor(0);

    if (panel == panelHandle[BCKGRND])
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz ZVB Network Analyzer\n"
                              "instrument driver provided in the package.\n");

    if (panel == panelHandle[INIT])
        MessagePopup ("Help", "This panel performs the following initialization actions:\n\n"
                              "- Opens a instrument session.\n\n"
                              "- Performs an identification query on the Instrument.\n\n"
                              "- Sends initialization commands to the instrument that set any\n"
                              "necessary programmatic variables to the state necessary\n"
                              "for the operation of the instrument driver.");
    
    if (panel == panelHandle[CONFIG])
        MessagePopup ("Help", "This panel configures the instrument.");
    
    if (panel == panelHandle[MEAS])
        MessagePopup ("Help", "This panel displays the measured values.\n"
                              "Configuration of the instrument is not\n"
                              "possible with this panel. Press the\n"
                              "\"Configure\" button to return to\n"
                              "the configuration panel.");

    return;
}

/*===========================================================================*/
/* Function: Launch Close Panel                                              */
/* Purpose:  This is a callback function of the button Close on the          */
/*           panel Measure. It pops-up a panel that will close the           */
/*           instrument.                                                     */
/*===========================================================================*/
int CVICALLBACK launchClose (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar errorMessage[256], errBuffer[256];

    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            runState = 0;
            if ((ConfirmPopup ("Exit Application", "Are you sure you want to quit "
                    "this application?")) == 1) {
                if ((error = rszvb_close (io)) < 0) {
                    rszvb_error_message(VI_NULL, error, errorMessage);
                    sprintf (errBuffer, "Application failed to close the "
                        "instrument.\n\n%s\n\nExiting Application.", errorMessage);
                    MessagePopup("ERROR!", errBuffer);
                }

                QuitUserInterface (uirCode);
                exit (-1);
            }
            runState = 1;
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will pop-up a panel to close the instrument.");
            break;
    }
    return 0;
}   

/*****************************************************************************/

long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];

    SetWaitCursor (0);
    rszvb_error_message(VI_NULL, error, errorMessage);
    sprintf (errBuffer,"The following error caused the application to fail:\n\n%s\n\n"
        "Attempting to Close Instrument Driver and Exit Application", 
        errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
    
    if ((error = rszvb_close (io)) < 0) {
        rszvb_error_message(VI_NULL, error, errorMessage);
        sprintf (errBuffer, "Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
    }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
        
    QuitUserInterface (uirCode);
    exit (-1);
    
    return 0;
}

/*= End =====================================================================*/
