#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "RSZVB_Mixer_Measurement_Example.h"
#include "rszvb.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rszvb_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rszvb_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rszvb_close(io); \
            break; \
        } \
    }
    
/* Globals */

ViInt32 	panelHandle;
ViSession 	io;
ViStatus    error;
ViChar      error_message[256];
ViChar      error_buffer[256];

/* Local Functions */
int initialize (void);
            
/*= Main Function ===========================================================*/   
int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    panelHandle = LoadPanel (0, "RSZVB_Mixer_Measurement_Example.uir", PANEL); 
    DisplayPanel (panelHandle);  
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}


/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{

    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz ZVB Vector Network Analyzer\n"
                              "instrument driver provided in the package.\n");

    return;
}
                     
               
/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    
    ViReal64 startPow, stopPow, cwPow, startFreq, stopFreq, cwFreq, fixPow, fixFreq;
    ViInt32 sweepType, loSource, rfPower, rfFreq, loPow, loFreq, ifFreq, conversion;
    
    switch (event)
    {
        case EVENT_COMMIT:
            
            GetCtrlVal (panelHandle,PANEL_SWEEPTYPE    , &sweepType);
            GetCtrlVal (panelHandle,PANEL_LOSOURCE     , &loSource);
            GetCtrlVal (panelHandle,PANEL_STARTPOW     , &startPow);
            GetCtrlVal (panelHandle,PANEL_STOPPOW      , &stopPow);
            GetCtrlVal (panelHandle,PANEL_CWPOW        , &cwPow);
            GetCtrlVal (panelHandle,PANEL_STARTFREQ    , &startFreq);
            GetCtrlVal (panelHandle,PANEL_STOPFREQ     , &stopFreq);
            GetCtrlVal (panelHandle,PANEL_CWFREQ       , &cwFreq);
            GetCtrlVal (panelHandle,PANEL_FIXPOW       , &fixPow);    
            GetCtrlVal (panelHandle,PANEL_FIXFREQ      , &fixFreq);
            GetCtrlVal (panelHandle,PANEL_RFPOW        , &rfPower);
            GetCtrlVal (panelHandle,PANEL_LOPOW        , &loPow);
            GetCtrlVal (panelHandle,PANEL_RFFREQ       , &rfFreq);
            GetCtrlVal (panelHandle,PANEL_LOFREQ       , &loFreq);
            GetCtrlVal (panelHandle,PANEL_IFFREQ       , &ifFreq);
            GetCtrlVal (panelHandle,PANEL_CONVERSION   , &conversion);

       	    if (rfFreq == ifFreq)
			{
				MessagePopup ("RF and IF Frequency","Incompatible frequecy setting");
				return 1;
			}
			else if (rfFreq==loFreq)
			{
                MessagePopup ("RF and LO Frequency","Incompatible frequecy setting");
				return 1;
			}
			else if (loFreq==ifFreq)
			{
				MessagePopup ("LO and IF Frequency","Incompatible frequecy setting");
				return 1;
			}
            
			if (loPow == rfPower)
			{
				MessagePopup ("LO and RF Power","Incompatible power setting");  
            	return 1;
			}
           
			SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
            ProcessDrawEvents ();
            
			if (initialize() < VI_SUCCESS)
            {
                SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
                break;
            }
            
			SetWaitCursor(1);
			
			CHECKERR (rszvb_SetSweepType (io, 1, sweepType));
            
            //error 
            CHECKERR (rszvb_SetMixerMode (io, 1, RSZVB_MIX_MODE_MIXER));
            CHECKERR (rszvb_SetInternalSignalSource (io, 1, loSource));
   			
			
            switch (rfPower)
			{
			    case 0:
					    CHECKERR (rszvb_ConfigurePowerSettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_RF, fixPow));
			    break;
				
				case 1:
					    CHECKERR (rszvb_ConfigurePowerSettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_LO, fixPow));
				break;
				
	        }
			  
			switch (rfFreq)
			{
			    case 0:	
                        if (loFreq == 1)
						{	
					    	CHECKERR (rszvb_ConfigureFrequencySettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_RF,
														RSZVB_FUNDAMENTAL_TYPE_LO, fixFreq,
														conversion));
						}
						else
						{
							CHECKERR (rszvb_ConfigureFrequencySettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_RF,
														RSZVB_FUNDAMENTAL_TYPE_IF, fixFreq,
														conversion));
						}
			    break;
				
				case 1:
						if (loFreq == 0)
						{
					    	CHECKERR (rszvb_ConfigureFrequencySettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_LO,
														RSZVB_FUNDAMENTAL_TYPE_RF, fixFreq,
														conversion));
						}
						else
						{
							CHECKERR (rszvb_ConfigureFrequencySettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_IF,
														RSZVB_FUNDAMENTAL_TYPE_RF, fixFreq,
														conversion));
						}
				break;
				
				case 2:	
					    if (loFreq == 0)
						{
							CHECKERR (rszvb_ConfigureFrequencySettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_LO,
														RSZVB_FUNDAMENTAL_TYPE_IF, fixFreq,
														conversion));
						}
						else
						{
							CHECKERR (rszvb_ConfigureFrequencySettings (io, 1, RSZVB_FUNDAMENTAL_TYPE_IF,
														RSZVB_FUNDAMENTAL_TYPE_LO, fixFreq,
														conversion));	
						}
				break;	   
			}		
					
            switch (sweepType)
			{
                case  RSZVB_SWEEP_LIN:
                        CHECKERR (rszvb_SetPower (io, 1, cwPow));
                        CHECKERR (rszvb_SetStartFrequency (io, 1, startFreq));
                        CHECKERR (rszvb_SetStopFrequency (io, 1, stopFreq));
                break;
        
                case  RSZVB_SWEEP_POW:
                        CHECKERR (rszvb_SetCWFrequency (io, 1, cwFreq));
                        CHECKERR (rszvb_SetStartPower (io, 1, startPow));
                        CHECKERR (rszvb_SetStopPower (io, 1, stopPow));
                break;
        
                case  RSZVB_SWEEP_CW:
                        CHECKERR (rszvb_SetCWFrequency (io, 1, cwFreq)); 
                        CHECKERR (rszvb_SetPower (io, 1, cwPow));
                break;
            }    

            rszvb_close(io);
            
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
            SetWaitCursor(0);
            
			break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Sweep Type                                                      */
/* Purpose:  Enables or disables sets of values for selected sweep type.     */
/*===========================================================================*/
int CVICALLBACK sweepType(int panel, int control, int event, void *callbackData,
        int eventData1, int eventData2)
{
    ViInt32 sweep_type = 0;
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetCtrlVal (panelHandle, PANEL_SWEEPTYPE, &sweep_type);
            if (sweep_type == RSZVB_SWEEP_LIN)
                {
                SetCtrlAttribute (panelHandle, PANEL_STARTPOW, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_STOPPOW, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_CWPOW, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_STARTFREQ, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_STOPFREQ, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_CWFREQ, ATTR_DIMMED, 1);
                }
            else if (sweep_type == RSZVB_SWEEP_POW)
                {
                SetCtrlAttribute (panelHandle, PANEL_STARTPOW, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_STOPPOW, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_CWPOW, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_STARTFREQ, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_STOPFREQ, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_CWFREQ, ATTR_DIMMED, 0);
                }
			else if (sweep_type == RSZVB_SWEEP_CW)
                {
                SetCtrlAttribute (panelHandle, PANEL_STARTPOW, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_STOPPOW, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_CWPOW, ATTR_DIMMED, 0);
                SetCtrlAttribute (panelHandle, PANEL_STARTFREQ, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_STOPFREQ, ATTR_DIMMED, 1);
                SetCtrlAttribute (panelHandle, PANEL_CWFREQ, ATTR_DIMMED, 0);
                }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help", "Selects the sweep type, i.e. the sweep variable (frequency/power)\n"
                                  " and the position of the sweep points across the sweep range.\n\n"
								  "Valid Range:\n"
								  "0 - Frequency\n"
								  "1 - Power\n"
								  "2 - CW Mode (Default Value)");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls that configure the  */
/*           instrument. On the right mouse-click on the control a help      */
/*           window describing its purpose is displayed.                     */
/*===========================================================================*/
int CVICALLBACK controlHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{   
    SetWaitCursor(0);

    switch (event)
        {
        case EVENT_RIGHT_CLICK:
            
                switch (control)
                    {
                    case PANEL_INSTR_DESC:
                        MessagePopup ("Help", "This control specifies the interface and address of the device that\n"
                                              "is to be initialized (Instrument Descriptor). The exact grammar to\n"
                                              "be used in this control is shown in the note below.\n\n"
                                              "Default Value: GPIB::20\n\n"
                                              "Notes:\n\n"
                                              "(1) Based on the Instrument Descriptor, this operation\n"
                                              "establishes a communication session with a device.  The grammar\n"
                                              "for the Instrument Descriptor is shown below.  Optional\n"
                                              "parameters are shown in square brackets ([]).\n\n"
                                              "Interface   Grammar\n"
                                              "------------------------------------------------------\n"
                                              "Serial      ASRL<port>[::INSTR]\n"
                                              "GPIB        GPIB[board]::primary address[::secondary address]\n"
                                              "            [::INSTR]\n"
                                              "RSIB        RSIB::remote_host::INSTR\n"
                                              "VXI-11      TCPIP[board]::host address[::LAN device name][::INSTR]\n\n"
                                              "The ASRL keyword is used for serial interface.\n"
                                              "The GPIB keyword is used for GPIB interface.\n"
                                              "The RSIB keyword is used for RSIB interface.\n"
                                              "The VXI-11 keyword is used for a TCP/IP device located at the specified address.\n\n"
                                              "Examples:\n"
                                              "(1) Serial - ASRL1\n"
                                              "(2) GPIB   - GPIB::20\n"
                                              "(3) RSIB   - RSIB::192.168.1.33::INSTR\n"
                                              "(4) VXI-11 - TCPIP::10.0.0.123::INSTR\n\n"
                                              "The default value for optional parameters are shown below.\n\n"
                                              "Optional Parameter          Default Value\n"
                                              "-----------------------------------------\n"
                                              "board                       0\n"
                                              "secondary address           none - 31\n\n"
                                              "Note:\n\n"
                                              "(1) Parameters of the serial interface, you must set via T&M\n"
                                              "Expolrer before you call initialize function. Default values are\n"
                                              "Baud Rate: 9600, Data Bits: 8, Parity: None, Stop Bits: 1 and\n"
                                              "Flow Control: None");
                        break;
                    case PANEL_IDQUERY:
                        MessagePopup ("Help", "This control specifies if an ID Query is sent to the\n"
                                              "instrument during the initialization procedure.\n\n"
                                              "Valid Range:\n"
                                              "0 - Skip Query\n"
                                              "1 - Do Query (Default Value)");
                        break;
                    case PANEL_RESET:
                        MessagePopup ("Help", "This control specifies if the instrument is to be reset to its\n"
                                              "power-on settings during the initialization procedure.\n\n"
                                              "Valid Range:\n"
                                              "0 - Don't Reset\n"
                                              "1 - Reset Device (Default Value)");
                        break;
                    case PANEL_CWPOW:
                        MessagePopup ("Help", "This control defines the power of the internal signal source.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case PANEL_STOPFREQ:
                        MessagePopup ("Help", "This control defines the stop frequency for a frequency sweep\n"
                                              "which is equal to the right edge of a Cartesian diagram.\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    case PANEL_STARTFREQ:
                        MessagePopup ("Help", "This control defines the start frequency for a frequency sweep\n"
                                              "which is equal to the left edge of a Cartesian diagram.\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    case PANEL_FIXPOW:
                        MessagePopup ("Help", "This control defines a fixed power, to be assigned either to the RF\n"
                                              "or to the LO signal.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case PANEL_FIXFREQ:
                        MessagePopup ("Help", "This control defines a fixed frequency, to be assigned either to the\n"
                                              "RF, the LO, or the IF signal.\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    case PANEL_CWFREQ:
                        MessagePopup ("Help", "This control defines the fixed (Continuous Wave, CW) frequency\n"
                                              "for all sweep types operating at fixed frequency (power sweep,\n"
											  "time sweep, CW mode sweep).\n\n"
                                              "Valid Range: 0.0 to 8.0e9 (depends on the model)");
                        break;
                    case PANEL_STOPPOW:
                        MessagePopup ("Help", "This control defines the stop power for a power sweep which is\n"
							                  "equal to the right edge of a Cartesian diagram.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case PANEL_STARTPOW:
                        MessagePopup ("Help", "This control defines the start power for a power sweep which is \n"
                                              "equal to the left edge of a Cartesian diagram.\n\n"
                                              "Valid Range: -40.0 to 10.0 dBm");
                        break;
                    case PANEL_LOSOURCE:
                        MessagePopup ("Help", "This control sets an analyzer port as a signal source\n"
                                              "for the LO signal (internal source).\n\n"
                                              "Valid Range:\n"
											  "0 - (NONE) (Default Value)\n"
											  "Port 2 ZVB\n"
											  "Port 3 ZVA/ZVT\n"
											  "Port 4 ZVA/ZVB/ZVT");
                        break;
                    case PANEL_LOPOW:
                        MessagePopup ("Help", "This control specifies how the power for each signal is defined. The\n"
                                              "roles of the mixer input signals RF and LO are interchangeable. \n\n"
                                              "Valid Range: \n"
						                      "0 - CW/Swept\n"
										      "1 - Fixed (Default Value)");
                        break;
				    case PANEL_IFFREQ:
                        MessagePopup ("Help", "This control specifies how the frequency for each signal is defined.\n"
                                              "The roles of the three signals RF, IF, LO are interchangeable.\n\n"
                                              "Valid Range:\n"
											  "0 - CW/Swept\n"
											  "1 - Fixed\n"
											  "2 - Auto (Default Value)");
                        break;
                    case PANEL_CONVERSION:
                        MessagePopup ("Help", "This control selects the frequency conversion mode of the IF signal.\n\n"
                                              "Valid Range:\n"
											  "0 - IF = LO  RF (Default Value)\n"
											  "1 - IF = RF  LO\n"
											  "2 - IF = LO + RF");
                        break;
                    case PANEL_LOFREQ:
                        MessagePopup ("Help", "This control specifies how the frequency for each signal is defined.\n"
                                              "The roles of the three signals RF, IF, LO are interchangeable.\n\n"
                                              "Valid Range:\n"
											  "0 - CW/Swept\n"
											  "1 - Fixed (Default Value)\n"
											  "2 - Auto");
                        break;
                    case PANEL_RFFREQ:
                        MessagePopup ("Help", "This control specifies how the frequency for each signal is defined.\n"
                                              "The roles of the three signals RF, IF, LO are interchangeable.\n\n"
                                              "Valid Range:\n"
											  "0 - CW/Swept (Default Value)\n"
											  "1 - Fixed\n"
											  "2 - Auto");
                        break;
                    case PANEL_RFPOW:
                        MessagePopup ("Help", "This control specifies how the power for each signal is defined. The\n"
                                              "roles of the mixer input signals RF and LO are interchangeable. \n\n"
                                              "Valid Range: \n"
						                      "0 - CW/Swept (Default Value)\n"
										      "1 - Fixed");
                        break;
                    }

            break;
        }
    
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;
    

    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rszvb_init (instr_desc, id, rst, &io)) < 0 )
    {
        rszvb_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }
	
    //CHECKERR (rszvb_SetDisplayUpdate (io, RSZVB_DISP_UPDATE_ON));
	//CHECKERR (viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000));
    SetWaitCursor(0);
    return 0;
}
